library ieee;
library std;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

package test_jesd_util_pkg is

  procedure generate_inv_prbs_flex( constant i : in std_logic_vector(22 downto 0);
                                    constant num : in integer;
                                    constant pattern : in integer;
                                    variable q : out std_logic_vector(22 downto 0);
                                    variable d : out std_logic_vector            );

  procedure check_inv_prbs_flex( constant d  : in std_logic_vector;
                                 constant i  : in std_logic_vector(22 downto 0);
                                 constant num : in integer;
                                 constant pattern : in integer;
                                 variable q  : out std_logic_vector(22 downto 0);
                                 variable ok : out std_logic );

end test_jesd_util_pkg;

package body test_jesd_util_pkg is

  function second_adder (i : integer) return integer is
  begin
    case i is
      when 7 => return 5;
      when 9 => return 4;
      when 15 => return 13;
      when others => return 17;
    end case;
  end second_adder;

  procedure generate_inv_prbs_flex( constant i : in std_logic_vector(22 downto 0);
                                    constant num : in integer;
                                    constant pattern : in integer;
                                    variable q : out std_logic_vector(22 downto 0);
                                    variable d : out std_logic_vector ) is
    variable p : std_logic_vector(22 downto 0);
    variable x : std_logic;
  begin
    p := i;
    for n in d'high downto (d'high+1-num) loop
      x := p(pattern-1) xor p(second_adder(pattern));
      p(pattern-1 downto 0) := p(pattern-2 downto 0) & x;
      if pattern=23 then
        --d(n) := not x;
        d(n) := x;
      else
        d(n) := x;
      end if;
    end loop;
    q := p;
  end generate_inv_prbs_flex;

  procedure check_inv_prbs_flex( constant d  : in std_logic_vector;
                                  constant i  : in std_logic_vector(22 downto 0);
                                  constant num : in integer;
                                  constant pattern : in integer;
                                  variable q  : out std_logic_vector(22 downto 0);
                                  variable ok : out std_logic ) is
    variable p : std_logic_vector(22 downto 0);
    variable x : std_logic;
    variable t : std_logic;
    variable e : std_logic;
  begin
    p := i;
    t := '1';
    for n in d'high downto (d'high+1-num) loop
      if pattern=23 then
        --e := not d(n);
        e := d(n);
      else
        e := d(n);
      end if;
      x := p(pattern-1) xor p(second_adder(pattern));
      p(pattern-1 downto 0) := p(pattern-2 downto 0) & e;
      if e /= x then
        t := '0';
      end if;
    end loop;
    q := p;
    ok := t;
  end check_inv_prbs_flex;

end ;
