library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all ;
use ieee.std_logic_unsigned.all ;
use work.ieee_ext.all ;

library altera_mf;
use altera_mf.altera_mf_components.all;

entity buf_ram is
  generic (
    DW             : integer := 36;
    AW             : integer := 9;
    DEPTH          : integer := 0;
    RAM_BLOCK_TYPE : string := "AUTO" );
  port (
    wr_clk  : in  std_logic;
    wr_en   : in  std_logic;
    wr_addr : in  std_logic_vector(AW-1 downto 0);
    wr_data : in  std_logic_vector(DW-1 downto 0);
    rd_clk  : in  std_logic;
    rd_en   : in  std_logic;
    rd_addr : in  std_logic_vector(AW-1 downto 0);
    rd_data : out std_logic_vector(DW-1 downto 0) );
end buf_ram;

architecture synth of buf_ram is

  constant NUMWORDS : integer := cond_expr((DEPTH = 0), 2**AW, DEPTH);

  component altsyncram
    generic (
      operation_mode         :     string;
      clock_enable_input_a   :     string;
      clock_enable_input_b   :     string;
      clock_enable_output_b  :     string;
      outdata_reg_b	     :     string;
      width_a                :     natural;
      widthad_a              :     natural;
      numwords_a             :     natural;
      width_b                :     natural;
      widthad_b              :     natural;
      numwords_b             :     natural;
      lpm_type               :     string;
      ram_block_type         :     string );
    port (
      clock0                 : in  std_logic;
      wren_a                 : in  std_logic;
      address_a              : in  std_logic_vector (widthad_a-1 downto 0);
      data_a                 : in  std_logic_vector (width_a-1 downto 0);
      clock1                 : in  std_logic;
      clocken1		     : in  std_logic;
      address_b              : in  std_logic_vector (widthad_b-1 downto 0);
      q_b                    : out std_logic_vector (width_b-1 downto 0) );
  end component;
begin
  r : altsyncram
    generic map (
      operation_mode         => "DUAL_PORT",
      clock_enable_input_a   => "NORMAL",
      clock_enable_input_b   => "NORMAL",
      clock_enable_output_b  => "NORMAL",
      outdata_reg_b          => "UNREGISTERED",
      width_a                => DW,
      widthad_a              => AW,
      numwords_a             => NUMWORDS,
      width_b                => DW,
      widthad_b              => AW,
      numwords_b             => NUMWORDS,
      lpm_type               => "altsyncram",
      ram_block_type         => RAM_BLOCK_TYPE )
    port map (
      clock0    => wr_clk,
      wren_a    => wr_en,
      address_a => wr_addr,
      data_a    => wr_data,
      clock1    => rd_clk,
      clocken1  => rd_en,
      address_b => rd_addr,
      q_b       => rd_data );
end synth;
