library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.ieee_ext.all;
use work.jesd_eval_pkg.all;
use work.test_jesd_util_pkg.all;

entity jesd204b_eval_core is
  generic (
    SIMULATION         : boolean := false;
    NO_LANES           : integer range 1 to 8 := 4;
    NO_CONVERTERS      : integer := 4
  );
  port (
    clk_device         : in  std_logic;
	 s_clk              : in  std_logic;
    sync_b_out         : out std_logic;
    sync_b_in          : in  std_logic;
    sysref_in          : in  std_logic;
    sysref_out         : out std_logic;
    clk_ref            : in  std_logic_vector(NO_LANES-1 downto 0);                   -- Reference clock
    --clk_ref            : in  std_logic_vector(NO_LANES-1 downto 0);                   -- Reference clock
    rx_serial_data     : in  std_logic_vector(NO_LANES-1 downto 0);                   -- Serial receive data
--    tx_serial_data     : out std_logic_vector(NO_LANES-1 downto 0);                   -- Serial transmit data
    system_pll_locked  : in  std_logic;
    -- CPU clock/reset
    cpu_clk            : in  std_logic;
    cpu_reset          : in  std_logic;
    -- CPU interface
    cpu_address        : in  std_logic_vector(15 downto 0);                  -- CPU address
    cpu_wr_data        : in  std_logic_vector(31 downto 0);                  -- CPU write data
    cpu_rd_data        : out std_logic_vector(31 downto 0);                  -- CPU read data
    cpu_read           : in  std_logic;                                      -- CPU read
    cpu_write          : in  std_logic;                                      -- CPU write
    cpu_select         : in  std_logic;                                      -- CPU select
    cpu_waitrequest    : out std_logic;                                      -- CPU acknowledge
    cpu_intr           : out std_logic;                                      -- CPU interrupt
    -- ADI SPI i/f
    spi_di             : in  std_logic;
    spi_do             : out std_logic;
    spi_do_en          : out std_logic;
    spi_clk            : out std_logic;
    spi_cs_n           : out std_logic;
	 test_clk           : out std_logic;
    -- SFP+ status
    sfp_mod_abs        : in  std_logic_vector(NO_LANES-1 downto 0);                   -- Module present indicator
    sfp_op_los         : in  std_logic_vector(NO_LANES-1 downto 0);                   -- Signal present indicator
    sfp_op_tx_flt      : in  std_logic_vector(NO_LANES-1 downto 0);                    -- Transmitter fault indicator
	 trigger_in         : in std_logic
  );
end jesd204b_eval_core;

architecture rtl of jesd204b_eval_core is
-------------------------------------------------------------------------------
component serdes
	port (
		rx_analogreset      : in  std_logic_vector(0 downto 0)  := (others => '0'); --      rx_analogreset.rx_analogreset
		rx_digitalreset     : in  std_logic_vector(0 downto 0)  := (others => '0'); --     rx_digitalreset.rx_digitalreset
		rx_cdr_refclk       : in  std_logic_vector(0 downto 0)  := (others => '0'); --       rx_cdr_refclk.rx_cdr_refclk
		rx_pma_clkout       : out std_logic_vector(0 downto 0);                     --       rx_pma_clkout.rx_pma_clkout
		rx_serial_data      : in  std_logic_vector(0 downto 0)  := (others => '0'); --      rx_serial_data.rx_serial_data
		rx_set_locktodata   : in  std_logic_vector(0 downto 0)  := (others => '0'); --   rx_set_locktodata.rx_set_locktodata
		rx_set_locktoref    : in  std_logic_vector(0 downto 0)  := (others => '0'); --    rx_set_locktoref.rx_set_locktoref
		rx_is_lockedtoref   : out std_logic_vector(0 downto 0);                     --   rx_is_lockedtoref.rx_is_lockedtoref
		rx_is_lockedtodata  : out std_logic_vector(0 downto 0);                     --  rx_is_lockedtodata.rx_is_lockedtodata
		rx_parallel_data    : out std_logic_vector(63 downto 0);                    --    rx_parallel_data.rx_parallel_data
		rx_std_coreclkin    : in  std_logic_vector(0 downto 0)  := (others => '0'); --    rx_std_coreclkin.rx_std_coreclkin
		rx_std_clkout       : out std_logic_vector(0 downto 0);                     --       rx_std_clkout.rx_std_clkout
		rx_std_pcfifo_full  : out std_logic_vector(0 downto 0);                     --  rx_std_pcfifo_full.rx_std_pcfifo_full
		rx_std_pcfifo_empty : out std_logic_vector(0 downto 0);                     -- rx_std_pcfifo_empty.rx_std_pcfifo_empty
		rx_cal_busy         : out std_logic_vector(0 downto 0);                     --         rx_cal_busy.rx_cal_busy
		reconfig_to_xcvr    : in  std_logic_vector(69 downto 0) := (others => '0'); --    reconfig_to_xcvr.reconfig_to_xcvr
		reconfig_from_xcvr  : out std_logic_vector(45 downto 0)                     --  reconfig_from_xcvr.reconfig_from_xcvr
	);
end component;
      
component serdes_reconfig
  port (		
		reconfig_busy             : out std_logic;                                        --   reconfig_busy.reconfig_busy
		tx_cal_busy               : out std_logic;                                        --     tx_cal_busy.tx_cal_busy
		rx_cal_busy               : out std_logic;                                        --     rx_cal_busy.tx_cal_busy
		cal_busy_in               : in  std_logic                     := '0';             --     cal_busy_in.cal_busy_in
		mgmt_clk_clk              : in  std_logic                     := '0';             --    mgmt_clk_clk.clk
		mgmt_rst_reset            : in  std_logic                     := '0';             --  mgmt_rst_reset.reset
		reconfig_mgmt_address     : in  std_logic_vector(6 downto 0)  := (others => '0'); --   reconfig_mgmt.address
		reconfig_mgmt_read        : in  std_logic                     := '0';             --                .read
		reconfig_mgmt_readdata    : out std_logic_vector(31 downto 0);                    --                .readdata
		reconfig_mgmt_waitrequest : out std_logic;                                        --                .waitrequest
		reconfig_mgmt_write       : in  std_logic                     := '0';             --                .write
		reconfig_mgmt_writedata   : in  std_logic_vector(31 downto 0) := (others => '0'); --                .writedata
		ch0_0_to_xcvr             : out std_logic_vector(69 downto 0);                    --   ch0_0_to_xcvr.reconfig_to_xcvr
		ch0_0_from_xcvr           : in  std_logic_vector(45 downto 0) := (others => '0'); -- ch0_0_from_xcvr.reconfig_from_xcvr
		ch1_1_to_xcvr             : out std_logic_vector(69 downto 0);                    --   ch1_1_to_xcvr.reconfig_to_xcvr
		ch1_1_from_xcvr           : in  std_logic_vector(45 downto 0) := (others => '0'); -- ch1_1_from_xcvr.reconfig_from_xcvr
		ch2_2_to_xcvr             : out std_logic_vector(69 downto 0);                    --   ch2_2_to_xcvr.reconfig_to_xcvr
		ch2_2_from_xcvr           : in  std_logic_vector(45 downto 0) := (others => '0'); -- ch2_2_from_xcvr.reconfig_from_xcvr
		ch3_3_to_xcvr             : out std_logic_vector(69 downto 0);                    --   ch3_3_to_xcvr.reconfig_to_xcvr
		ch3_3_from_xcvr           : in  std_logic_vector(45 downto 0) := (others => '0')  -- ch3_3_from_xcvr.reconfig_from_xcvr
);
  end component;

------------------------------------------------------------------------------------------------
  
 
  
  
component data_fifo
	PORT
	(
		data		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
		rdclk		: IN STD_LOGIC ;
		rdreq		: IN STD_LOGIC ;
		wrclk		: IN STD_LOGIC ;
		wrreq		: IN STD_LOGIC ;
		q		   : OUT STD_LOGIC_VECTOR (15 DOWNTO 0);
		rdusedw	: OUT STD_LOGIC_VECTOR (9 DOWNTO 0)
	);
   end component;  
  
  
  
  
  
  
  
  
  component serdes_reset
  port (
    clock              : in  std_logic                    := '0';             --              clock.clk
    reset              : in  std_logic                    := '0';             --              reset.reset
    rx_analogreset     : out std_logic_vector(0 downto 0);                    --     rx_analogreset.rx_analogreset
    rx_digitalreset    : out std_logic_vector(0 downto 0);                    --    rx_digitalreset.rx_digitalreset
    rx_ready           : out std_logic_vector(0 downto 0);                    --           rx_ready.rx_ready
    rx_is_lockedtodata : in  std_logic_vector(0 downto 0) := (others => '0'); -- rx_is_lockedtodata.rx_is_lockedtodata
    rx_cal_busy        : in  std_logic_vector(0 downto 0) := (others => '0')  --        rx_cal_busy.rx_cal_busy
  );
  end component;

  component jesd_rx
    generic (
      NO_CONVERTERS       : in integer;
      NO_LANES            : in integer;
      NO_SERDES_WORDS     : in integer;
      NO_FRAME            : in integer);
    port (
      clk_device          : in  std_logic;
      reset_clk_device    : in  std_logic;
      clk_cpu             : in  std_logic;
      reset_clk_cpu       : in  std_logic;
      clk_serdes          : in  std_logic_vector(NO_LANES-1 downto 0);
      reset_clk_serdes    : in  std_logic_vector(NO_LANES-1 downto 0);
      cpu_address         : in  std_logic_vector(11 downto 0);
      cpu_wr_data         : in  std_logic_vector(31 downto 0);
      cpu_rd_data         : out std_logic_vector(31 downto 0);
      cpu_read            : in  std_logic;
      cpu_write           : in  std_logic;
      cpu_select          : in  std_logic;
      cpu_ack             : out std_logic;
      cpu_intr            : out std_logic;
      sample_data         : out std_logic_vector(16*NO_SERDES_WORDS*NO_CONVERTERS-1 downto 0);
      sample_control_bits : out std_logic_vector(3*NO_SERDES_WORDS*NO_CONVERTERS-1 downto 0);
      sample_valid        : out std_logic_vector(NO_CONVERTERS-1 downto 0);
      sample_error        : out std_logic;
      sysref              : in  std_logic;
      sync_b              : out std_logic;
      serdes_data         : in  std_logic_vector(10*NO_LANES*NO_SERDES_WORDS-1 downto 0));
  end component;


  

  constant SPI_OPERATION_WRITE : std_logic := '0';
  constant SPI_OPERATION_READ  : std_logic := '1';

  constant NO_SERDES_WORDS : integer := 4;
  constant NO_FRAME        : integer := 4;

  ----------------------------------------------------------------------------------------------------------------------
  -- Serdes interface signals
  ----------------------------------------------------------------------------------------------------------------------
  signal pll_powerdown        : std_logic_vector(NO_LANES-1 downto 0);
  signal tx_analogreset       : std_logic_vector(NO_LANES-1 downto 0);
  signal tx_digitalreset      : std_logic_vector(NO_LANES-1 downto 0);
  signal tx_parallel_data     : std_logic_vector((44*NO_LANES)-1 downto 0);
  signal pll_locked           : std_logic_vector(NO_LANES-1 downto 0);
  signal rx_analogreset       : std_logic_vector(NO_LANES-1 downto 0);
  signal rx_digitalreset      : std_logic_vector(NO_LANES-1 downto 0);
  signal rx_pma_clkout        : std_logic_vector(NO_LANES-1 downto 0);
  signal rx_parallel_data     : std_logic_vector((64*NO_LANES)-1 downto 0);
  signal rx_set_locktoref     : std_logic_vector(NO_LANES-1 downto 0);
  signal rx_set_locktodata    : std_logic_vector(NO_LANES-1 downto 0);
  signal rx_is_lockedtoref    : std_logic_vector(NO_LANES-1 downto 0);
  signal rx_is_lockedtodata   : std_logic_vector(NO_LANES-1 downto 0);
  signal tx_std_clkout        : std_logic_vector(NO_LANES-1 downto 0);
  signal rx_std_clkout        : std_logic_vector(NO_LANES-1 downto 0);

  signal tx_std_pcfifo_full   : std_logic_vector(NO_LANES-1 downto 0);
  signal tx_std_pcfifo_empty  : std_logic_vector(NO_LANES-1 downto 0);
  signal rx_std_pcfifo_full   : std_logic_vector(NO_LANES-1 downto 0);
  signal rx_std_pcfifo_empty  : std_logic_vector(NO_LANES-1 downto 0);

  signal tx_cal_busy          : std_logic_vector(NO_LANES-1 downto 0);
  signal rx_cal_busy          : std_logic_vector(NO_LANES-1 downto 0);

  type array140 is array(integer range <>) of std_logic_vector(69 downto 0);
  type array92  is array(integer range <>) of std_logic_vector(45 downto 0);

  -- Reconfiguration lanes for up to 8 lanes
  signal reconfig_to_xcvr     : array140(3 downto 0);
  signal reconfig_from_xcvr   : array92(3 downto 0);

  signal reconfig_busy        : std_logic;
  signal reconfig_tx_cal_busy : std_logic;
  signal reconfig_rx_cal_busy : std_logic;
  signal reconfig_cal_busy_in : std_logic;

  signal tx_ready             : std_logic_vector(NO_LANES-1 downto 0);
  signal pll_select           : std_logic_vector(NO_LANES-1 downto 0);
  signal rx_ready             : std_logic_vector(NO_LANES-1 downto 0);

  signal clk_frame           : std_logic;
  signal reset_clk_frame     : std_logic;


  ----------------------------------------------------------------------------------------------------------------------
  -- Sample RX interface
  ----------------------------------------------------------------------------------------------------------------------
  signal sample_rx_data_array   : array16(NO_SERDES_WORDS*NO_CONVERTERS-1 downto 0);
  signal sample_rx_ctrl_array   : array3(NO_SERDES_WORDS*NO_CONVERTERS-1 downto 0);

  signal sample_rx_data         : std_logic_vector(16*NO_SERDES_WORDS*NO_CONVERTERS-1 downto 0);
  signal sample_rx_control_bits : std_logic_vector(3*NO_SERDES_WORDS*NO_CONVERTERS-1 downto 0);
  signal sample_rx_valid        : std_logic_vector(NO_CONVERTERS-1 downto 0);
  signal sample_rx_error        : std_logic;

  signal sample_rx_check_clear  : std_logic;

  signal check_toggle    : std_logic;
  signal check_toggle_s0 : std_logic;
  signal check_toggle_s1 : std_logic;
  signal check_toggle_s2 : std_logic;

  signal sample_rx_check_error : std_logic_vector(NO_CONVERTERS-1 downto 0);
  signal sample_rx_check_valid : std_logic_vector(NO_CONVERTERS-1 downto 0);
  signal sample_rx_check_error_hold : std_logic_vector(NO_CONVERTERS-1 downto 0);
  signal sample_rx_check_valid_hold : std_logic_vector(NO_CONVERTERS-1 downto 0);
  signal sample_rx_check_error_hold_sync : std_logic_vector(NO_CONVERTERS-1 downto 0);
  signal sample_rx_check_valid_hold_sync : std_logic_vector(NO_CONVERTERS-1 downto 0);
  signal prbs_reg                     : array23(NO_CONVERTERS-1 downto 0);

  constant INCR          : std_logic_vector(2 downto 0) := "000";
  constant PRBS7         : std_logic_vector(2 downto 0) := "001";
  constant PRBS9         : std_logic_vector(2 downto 0) := "010";
  constant PRBS15        : std_logic_vector(2 downto 0) := "011";
  constant PRBS23        : std_logic_vector(2 downto 0) := "100";
  constant ADI           : std_logic := '0';
  signal sample_type     : std_logic_vector(2 downto 0);
  signal data_generator  : std_logic;

  ----------------------------------------------------------------------------------------------------------------------
  -- Sample TX interface
  ----------------------------------------------------------------------------------------------------------------------

  signal sample_check_incr      : std_logic_vector(8 downto 0);
  signal sample_tx_data         : std_logic_vector(16*NO_SERDES_WORDS*NO_CONVERTERS-1 downto 0);
  signal sample_tx_control_bits : std_logic_vector(3*NO_SERDES_WORDS*NO_CONVERTERS-1 downto 0);
  signal sample_tx_valid        : std_logic_vector(NO_CONVERTERS-1 downto 0);

  type sample_tx_data_type is array (integer range 0 to NO_SERDES_WORDS-1) of std_logic_vector(16*NO_CONVERTERS-1 downto 0);
  signal sample_tx_data_array : sample_tx_data_type;

  ----------------------------------------------------------------------------------------------------------------------
  -- Synchronization
  ----------------------------------------------------------------------------------------------------------------------
  signal sysref_en : std_logic;
  signal sysref_out_en : std_logic;
  signal sysref_toggle : std_logic;
  signal sysref_cnt_max : std_logic_vector(15 downto 0);

  signal local_sysref : std_logic;
  signal local_sync_b : std_logic;

  -------------------------------------------------------------------
  -- Serdes
  -------------------------------------------------------------------
  signal serdes_rx_data : std_logic_vector(10*NO_LANES*NO_SERDES_WORDS-1 downto 0);
  signal serdes_tx_data : std_logic_vector(10*NO_LANES*NO_SERDES_WORDS-1 downto 0);

  ----------------------------------------------------------------------------------------------------------------------
  -- SPI
  ----------------------------------------------------------------------------------------------------------------------
  signal spi_pending : std_logic;
  signal spi_done : std_logic;
  signal spi_operation : std_logic;
  signal spi_rate    : std_logic_vector(1 downto 0);
  signal spi_wr_data : std_logic_vector(7 downto 0);
  signal spi_rd_data : std_logic_vector(7 downto 0);
  signal spi_address : std_logic_vector(12 downto 0);

  ----------------------------------------------------------------------------------------------------------------------
  -- CAPTURE
  ----------------------------------------------------------------------------------------------------------------------
  signal cpu_capture_rd_data          : std_logic_vector(15 downto 0);
  signal capture_toggle               : std_logic;
  signal capture_active               : std_logic;
  signal capture_active_s0            : std_logic;
  signal capture_active_s1            : std_logic;

  ----------------------------------------------------------------------------------------------------------------------
  -- CPU
  ----------------------------------------------------------------------------------------------------------------------
  signal cpu_jesd204b_rx_rd_data      : std_logic_vector(31 downto 0);
  signal cpu_jesd204b_rx_select       : std_logic;
  signal cpu_jesd204b_rx_ack          : std_logic;
  signal cpu_jesd204b_rx_intr         : std_logic;

  signal cpu_jesd204b_tx_rd_data      : std_logic_vector(31 downto 0);
  signal cpu_jesd204b_tx_select       : std_logic;
  signal cpu_jesd204b_tx_ack          : std_logic;
  signal cpu_jesd204b_tx_intr         : std_logic;

  signal cpu_top_rd_data              : std_logic_vector(31 downto 0);
  signal cpu_top_select               : std_logic;
  signal cpu_top_ack                  : std_logic;
  signal cpu_top_read_done            : std_logic;
  signal cpu_top_write_done           : std_logic;
  signal cpu_wait_cnt                 : std_logic_vector(2 downto 0);

  signal cpu_reconfig_select          : std_logic;
  signal cpu_reconfig_read            : std_logic;
  signal cpu_reconfig_write           : std_logic;
  signal cpu_reconfig_rd_data         : std_logic_vector(31 downto 0);
  signal cpu_reconfig_rd_done         : std_logic;
  signal cpu_reconfig_ack             : std_logic;
  signal cpu_reconfig_waitreq         : std_logic;

  signal scratch_data                 : std_logic_vector(31 downto 0);
  signal serdes_rst                   : std_logic_vector(NO_LANES-1 downto 0);
  signal jesd_rx_reset                : std_logic;
  signal jesd_tx_reset                : std_logic;

  signal n_check                      : std_logic_vector(4 downto 0);

  -------------------------------------------------------------------
  -- Other signals
  -------------------------------------------------------------------
  signal cpu_waitrequest_mux          : std_logic;
  signal rx_seriallpbken              : std_logic_vector(NO_LANES-1 downto 0);
  signal serdes_status                : array11(NO_LANES-1 downto 0);
  signal cpu_reset_device             : std_logic;
  signal reset_device                 : std_logic;
  signal reset_device_s0              : std_logic;
  signal reset_device_s1              : std_logic;

  -------------------------------------------------------------------
  -- Debug signals
  -------------------------------------------------------------------
  signal cpu_clk_cnt                  : unsigned(31 downto 0);
  signal device_clk_cnt               : unsigned(31 downto 0);
  signal sigtap_debug                 : std_logic_vector(127 downto 0);

  signal sdata_ch0                    : std_logic_vector(15 downto 0);
  signal sdata_ch1                    : std_logic_vector(15 downto 0);
  signal sdata_ch2                    : std_logic_vector(15 downto 0);
  signal sdata_ch3                    : std_logic_vector(15 downto 0);
  signal rdusedw_sig0						  : std_logic_vector(9 downto 0);
  signal rdusedw_sig1						  : std_logic_vector(9 downto 0);
  signal rdusedw_sig2						  : std_logic_vector(9 downto 0);
  signal rdusedw_sig3						  : std_logic_vector(9 downto 0);

	 
  signal read_en                      : std_logic;
begin


-------------------------------------------------------------------------------------------
  gen_serdes:
  for i in 0 to NO_LANES-1 generate
    serdes_inst_0 : serdes
      port map (
		rx_analogreset(0)      => rx_analogreset(i),
		rx_digitalreset(0)     => rx_digitalreset(i),
		rx_cdr_refclk(0)       => clk_ref(i),
		rx_pma_clkout(0)       => rx_pma_clkout(i),
		rx_serial_data(0)      => rx_serial_data(i),
		rx_set_locktodata(0)   => rx_set_locktodata(i),
		rx_set_locktoref(0)    => rx_set_locktoref(i),
		rx_is_lockedtoref(0)   => rx_is_lockedtoref(i),
		rx_is_lockedtodata(0)  => rx_is_lockedtodata(i),
		rx_parallel_data    => rx_parallel_data(63+64*i downto 64*i),
		rx_std_coreclkin(0)    => rx_std_clkout(i),
		rx_std_clkout(0)       => rx_std_clkout(i),
		rx_std_pcfifo_full(0)  => rx_std_pcfifo_full(i),
		rx_std_pcfifo_empty(0) => rx_std_pcfifo_empty(i),
		rx_cal_busy(0)         => rx_cal_busy(i),
		reconfig_to_xcvr    => reconfig_to_xcvr(i),
		reconfig_from_xcvr  => reconfig_from_xcvr(i)
	);
end generate;

test_clk <= rx_std_clkout(0);

------------------------------------------------------------------------------------------
  -- Status
  process(pll_powerdown, pll_locked, tx_analogreset, tx_digitalreset, rx_analogreset, rx_digitalreset,
          rx_is_lockedtoref, rx_is_lockedtodata, rx_seriallpbken, tx_cal_busy, rx_cal_busy)
  begin
    for i in 0 to NO_LANES-1 loop
      serdes_status(i)(0)  <= pll_powerdown(i);
      serdes_status(i)(1)  <= pll_locked(i);
      serdes_status(i)(2)  <= tx_analogreset(i);
      serdes_status(i)(3)  <= tx_digitalreset(i);
      serdes_status(i)(4)  <= rx_analogreset(i);
      serdes_status(i)(5)  <= rx_digitalreset(i);
      serdes_status(i)(6)  <= rx_is_lockedtoref(i);
      serdes_status(i)(7)  <= rx_is_lockedtodata(i);
      serdes_status(i)(8)  <= rx_seriallpbken(i);
      serdes_status(i)(9)  <= tx_cal_busy(i);
      serdes_status(i)(10) <= rx_cal_busy(i);
    end loop;
  end process;


  -- Note: tx_std_coreclkin = tx_std_clkout and rx_std_coreclkin = rx_std_clkout;
  -- No clock compensation etc. used, see "Transceiver Custom Configurations in Arria V Devices" section 5-11 (http://www.altera.com/literature/hb/arria-v/av_53005.pdf)

  pll_select <= (others => '0');
  pll_powerdown <= (others => '0');

  
   
  gen_serdes_reset:
  for i in 0 to NO_LANES-1 generate
  serdes_reset_inst: serdes_reset
    port map (
      clock                 => clk_ref(i),
      reset                 => serdes_rst(i),
      rx_analogreset(0)     => rx_analogreset(i),
      rx_digitalreset(0)    => rx_digitalreset(i),
      rx_ready(0)           => rx_ready(i),
      rx_is_lockedtodata(0) => rx_is_lockedtodata(i),
      rx_cal_busy(0)        => rx_cal_busy(i)
    );
  end generate;

  process(cpu_clk,cpu_reset)
  begin
    if cpu_reset='1' then
      cpu_reconfig_rd_done <= '0';
    elsif cpu_clk'event and cpu_clk='1' then
      cpu_reconfig_rd_done <= (cpu_reconfig_read and not cpu_reconfig_waitreq) or (cpu_reconfig_rd_done and cpu_read);
    end if;
  end process;

  cpu_reconfig_ack <= cpu_reconfig_rd_done or not cpu_reconfig_read;

  reconfig_cal_busy_in <= '0'; -- See page 18 in an676. Tie to ground for only 1 reconfiguration controller.

 

--------------------------------------------------------------------------------
 -- Reconfig -> 4x SerDes
  -- Generate another one for another number of xcvr's
  -- Do not use for simulation, cailbration takes forever - literally.
--   gen_reconfig:
--   if not SIMULATION generate
     serdes_reconfig_inst: serdes_reconfig
       port map (
			reconfig_busy             => reconfig_busy,
			tx_cal_busy               => reconfig_tx_cal_busy,
			rx_cal_busy               => reconfig_rx_cal_busy,
			cal_busy_in               => reconfig_cal_busy_in,
			mgmt_clk_clk              => cpu_clk,
			mgmt_rst_reset            => cpu_reset,
			reconfig_mgmt_address     => cpu_address(6 downto 0),
			reconfig_mgmt_read        => cpu_reconfig_read,
			reconfig_mgmt_readdata    => cpu_reconfig_rd_data,
			reconfig_mgmt_waitrequest => cpu_reconfig_waitreq,
			reconfig_mgmt_write       => cpu_reconfig_write,
			reconfig_mgmt_writedata   => cpu_wr_data,
			ch0_0_to_xcvr             => reconfig_to_xcvr(0),               -- reconfig_to_xcvr
			ch0_0_from_xcvr           => reconfig_from_xcvr(0),
			ch1_1_to_xcvr             => reconfig_to_xcvr(1),               -- reconfig_to_xcvr              
			ch1_1_from_xcvr           => reconfig_from_xcvr(1),
			ch2_2_to_xcvr             => reconfig_to_xcvr(2),               -- reconfig_to_xcvr                 
			ch2_2_from_xcvr           => reconfig_from_xcvr(2),
			ch3_3_to_xcvr             => reconfig_to_xcvr(3),               -- reconfig_to_xcvr
			ch3_3_from_xcvr           => reconfig_from_xcvr(3)
		);





















  -- NEW Byte Serializer
  -- Info: serdes: tx_parallel_data: For each 44 bit word  the 40 active data bits are tx_parallel_data[42:33,31:22,20:11,9:0];
  -- Info: serdes: rx_parallel_data: For each 64 bit word  the 40 active data bits are rx_parallel_data[57:48,41:32,25:16,9:0];

  -- Remark that swapping of data words.
  --
  -- NO_LANES=4 and NO_SERDES_WORDS=2
  --
  -- rx_parallel_data
  --
  -- Lane 0 =    63:0 : w0 = 9:0 , w1 = 25:16 , TX order = 0 to 9 , 16 to 25
  -- Lane 1 =  127:64 :
  -- Lane 2 = 195:128 :
  -- Lane 3 = 255:196 :
  --
  -- serdes_rx_data / serdes_tx_data
  --
  -- Lane 0 =  19:0 : w0 = 19:10 , w1 = 9:0 , TX order = 10 to 19 , 0 to 9
  -- Lane 1 = 39:20 :
  -- Lane 2 = 59:40 :
  -- Lane 3 = 79:60 :

-- serdes_rx_data <= serdes_tx_data;

  process(rx_parallel_data)
  begin
    for lane in 0 to NO_LANES-1 loop
      for word in 0 to NO_SERDES_WORDS-1 loop
        serdes_rx_data((lane*NO_SERDES_WORDS+word+1)*10-1 downto (lane*NO_SERDES_WORDS+word)*10) <=
          rx_parallel_data(9+lane*64+(NO_SERDES_WORDS-1-word)*16 downto lane*64+(NO_SERDES_WORDS-1-word)*16);
      end loop;
    end loop;
  end process;


  jesd_rx_inst : jesd_rx
    generic map (
      NO_CONVERTERS       => NO_CONVERTERS,
      NO_LANES            => NO_LANES,
      NO_SERDES_WORDS     => NO_SERDES_WORDS,
      NO_FRAME            => NO_FRAME)
    port map (
      -- CLK/RESET
      clk_device          => clk_device,
      reset_clk_device    => reset_device,
      clk_cpu             => cpu_clk,
      reset_clk_cpu       => jesd_rx_reset,
      clk_serdes          => rx_std_clkout,
      reset_clk_serdes    => rx_digitalreset,
      -- CPU
      cpu_address         => cpu_address(11 downto 0),
      cpu_wr_data         => cpu_wr_data,
      cpu_rd_data         => cpu_jesd204b_rx_rd_data,
      cpu_read            => cpu_read,
      cpu_write           => cpu_write,
      cpu_select          => cpu_jesd204b_rx_select,
      cpu_ack             => cpu_jesd204b_rx_ack,
      cpu_intr            => cpu_jesd204b_rx_intr,
      -- SAMPLE
      sample_data         => sample_rx_data,
      sample_control_bits => sample_rx_control_bits,
      sample_valid        => sample_rx_valid,
      sample_error        => sample_rx_error,
      -- Synchronization
      sysref              => local_sysref,
      sync_b              => local_sync_b,
      -- SERDES
      serdes_data         => serdes_rx_data );
------------------------------------------------------------------------------------------------------
--Move data from 64 bits to 16 bits
------------------------------------------------------------------------------------------------------
data_fifo_inst1 : data_fifo PORT MAP (
		data	 => sample_rx_data(63 downto 0),
		rdclk	 => s_clk,
		rdreq	 => read_en,
		wrclk	 => clk_device,
		wrreq	 => sample_rx_valid(0) ,
		q	    => sdata_ch0(15 downto 0),
		rdusedw	 => rdusedw_sig0
	);

data_fifo_inst2 : data_fifo PORT MAP (
		data	 => sample_rx_data(127 downto 64),
		rdclk	 => s_clk,
		rdreq	 => read_en,
		wrclk	 => clk_device,
		wrreq	 => sample_rx_valid(1) ,
		q	    => sdata_ch1(15 downto 0),
		rdusedw	 => rdusedw_sig1
	);

data_fifo_inst3 : data_fifo PORT MAP (
		data	 => sample_rx_data(191 downto 128),
		rdclk	 => s_clk,
		rdreq	 => read_en,
		wrclk	 => clk_device,
		wrreq	 => sample_rx_valid(2) ,
		q	    => sdata_ch2(15 downto 0),
		rdusedw	 => rdusedw_sig2
	);

data_fifo_inst4 : data_fifo PORT MAP (
		data	 => sample_rx_data(255 downto 192),
		rdclk	 => s_clk,
		rdreq	 => read_en,
		wrclk	 => clk_device,
		wrreq	 => sample_rx_valid(3) ,
		q	    => sdata_ch3(15 downto 0),
		rdusedw	 => rdusedw_sig3
	);	

process (s_clk,rx_digitalreset(0))
begin
 if (rx_digitalreset(0)='1') then
  read_en <= '0';  --default state on reset.
elsif (rising_edge(s_clk)) then
  if rdusedw_sig0 >= "0001000000" then
       read_en <= '1'; 
   else 
	     read_en <= read_en; 
   
end if;
end if;
end process;




  -----------------------------------------------------------------------------
  -- CPU interface arbitration between JESD204B_RX, RECONFIG and TOP
  -----------------------------------------------------------------------------
  cpu_jesd204b_rx_select <= cpu_select and cpu_address(15 downto 12) = "0000";
  cpu_jesd204b_tx_select <= cpu_select and cpu_address(15 downto 12) = "0001";
  cpu_top_select         <= cpu_select and cpu_address(15 downto 12) = "0010";
  cpu_reconfig_select    <= cpu_select and cpu_address(15 downto 12) = "0011";

  cpu_reconfig_read      <= cpu_reconfig_select and cpu_read and not cpu_reconfig_rd_done;
  cpu_reconfig_write     <= cpu_reconfig_select and cpu_write;

  cpu_rd_data <= cpu_jesd204b_rx_rd_data when cpu_jesd204b_rx_select = '1' else
                 cpu_jesd204b_tx_rd_data when cpu_jesd204b_tx_select = '1' else
                 cpu_top_rd_data when cpu_top_select = '1' else
                 cpu_reconfig_rd_data when cpu_reconfig_select = '1' else
                 X"DEADBEEF";

  cpu_waitrequest_mux <= not cpu_jesd204b_rx_ack when cpu_jesd204b_rx_select = '1' else
             not cpu_jesd204b_tx_ack when cpu_jesd204b_tx_select = '1' else
             not cpu_top_ack when cpu_top_select = '1' else
             not cpu_reconfig_ack when cpu_reconfig_select='1' else
             '1';
  cpu_waitrequest <= cpu_waitrequest_mux;

  cpu_intr <= cpu_jesd204b_rx_intr;

  -----------------------------------------------------------------------------
  -- CPU TOP
  -----------------------------------------------------------------------------
  process(clk_device,reset_device)
  begin
    if reset_device=RESET_ACTIVE then
      check_toggle_s0 <= '0';
      check_toggle_s1 <= '0';
      check_toggle_s2 <= '0';
      sample_rx_check_error_hold <= (others => '0');
      sample_rx_check_valid_hold <= (others => '0');
      sample_rx_check_error_hold_sync <= (others => '0');
      sample_rx_check_valid_hold_sync <= (others => '0');
    elsif clk_device'event and clk_device='1' then
      check_toggle_s0 <= check_toggle;
      check_toggle_s1 <= check_toggle_s0;
      check_toggle_s2 <= check_toggle_s1;
      if check_toggle_s1 /= check_toggle_s2 then
        sample_rx_check_error_hold_sync <= sample_rx_check_error_hold;
        sample_rx_check_valid_hold_sync <= sample_rx_check_valid_hold;
        sample_rx_check_error_hold <= sample_rx_check_error;
        sample_rx_check_valid_hold <= sample_rx_check_valid;
      else
        sample_rx_check_error_hold <= sample_rx_check_error_hold or sample_rx_check_error;
        sample_rx_check_valid_hold <= sample_rx_check_valid_hold or sample_rx_check_valid;
      end if;

    end if;
  end process;

  process(cpu_clk,cpu_reset)
  begin
    if cpu_reset='1' then
      cpu_top_ack        <= '0';
      cpu_top_read_done  <= '0';
      cpu_top_write_done <= '0';
      cpu_top_rd_data    <= (others => '0');
      check_toggle       <= '0';
      cpu_wait_cnt       <= (others => '0');
      spi_rate           <= (others => '0');
      spi_wr_data        <= (others => '0');
      spi_address        <= (others => '0');
      spi_operation      <= '0';
      spi_pending        <= '0';
      rx_seriallpbken    <= (others => '1');
      rx_set_locktoref   <= (others => '0');
      rx_set_locktodata  <= (others => '0');
      cpu_reset_device   <= RESET_ACTIVE;
      serdes_rst         <= (others => '1');
      jesd_rx_reset      <= '1';
      jesd_tx_reset      <= '1';
      capture_active_s0  <= '0';
      capture_active_s1  <= '0';
      capture_toggle     <= '0';
      sysref_cnt_max     <= (others => '0');
      sysref_en          <= '0';
      sysref_out_en      <= '0';
      sysref_toggle      <= '0';
      n_check            <= conv_std_logic_vector(16, n_check'length);
      sample_type        <= INCR;
      data_generator     <= not ADI;

    elsif cpu_clk'event and cpu_clk='1' then
      cpu_top_read_done <= cpu_top_select and cpu_read;
      cpu_top_write_done <= cpu_top_select and cpu_write;
      cpu_top_ack <= cpu_top_select
                     and (cpu_read or cpu_write)
                     and (cpu_address(11 downto 0)/=REG_TOP_CHECK or cpu_wait_cnt="111")
                     and (cpu_address(11 downto 10)/=REG_TOP_CAPTURE_RX(11 downto 10) or cpu_wait_cnt="111")
                     and (cpu_address(11 downto 10)/=REG_TOP_CAPTURE_TX(11 downto 10) or cpu_wait_cnt="111");
      if cpu_top_select='1' and cpu_read='1' and cpu_top_read_done='0' and cpu_address(11 downto 0)=REG_TOP_CHECK then
        check_toggle <= not check_toggle;
      end if;
      if cpu_top_select='1' and (cpu_read='1' or cpu_write='1') then
        if cpu_wait_cnt/="111" then
          cpu_wait_cnt <= cpu_wait_cnt + "1";
        end if;
      else
        cpu_wait_cnt <= (others => '0');
      end if;
      capture_active_s0 <= capture_active;
      capture_active_s1 <= capture_active_s0;
      --! CPU READ !--
      if cpu_top_select='1' and cpu_read='1' and (cpu_top_read_done='0' or cpu_wait_cnt="111") then
        cpu_top_rd_data <= (others => '0');

        if cpu_address(11 downto 0)=REG_TOP_CHECK then
          for i in NO_CONVERTERS-1 downto 0 loop
            cpu_top_rd_data(2*i) <= sample_rx_check_valid_hold_sync(i);
            cpu_top_rd_data(2*i+1) <= sample_rx_check_error_hold_sync(i);
          end loop;
        end if;

        if cpu_address(11 downto 0)=REG_TOP_SAMPLE_CTRL then
          cpu_top_rd_data(2 downto 0) <= sample_type;
          cpu_top_rd_data(8)          <= data_generator;
        end if;

        if cpu_address(11 downto 0)=REG_TOP_SPI_STATUS then
          cpu_top_rd_data(0) <= spi_pending or spi_done;
        end if;

        if cpu_address(11 downto 0)=REG_TOP_SPI_RATE then
          cpu_top_rd_data(spi_rate'range) <= spi_rate;
        end if;

        if cpu_address(11 downto 0)=REG_TOP_SPI_CONTROL then
          if spi_operation=SPI_OPERATION_WRITE then
            cpu_top_rd_data(7 downto 0) <= spi_wr_data;
          elsif spi_pending='0' then
            cpu_top_rd_data(7 downto 0) <= spi_rd_data;
          else
            cpu_top_rd_data(7 downto 0) <= (others => '0');
          end if;
          cpu_top_rd_data(20 downto 8) <= spi_address;
          cpu_top_rd_data(24) <= spi_operation;
        end if;

        if cpu_address(11 downto 0)=REG_TOP_SFP_STATUS then
          for i in 0 to NO_LANES-1 loop
            cpu_top_rd_data(0+3*i) <= sfp_mod_abs(i);
            cpu_top_rd_data(1+3*i) <= sfp_op_los(i);
            cpu_top_rd_data(2+3*i) <= sfp_op_tx_flt(i);
          end loop;
        end if;

        if cpu_address(11 downto 0)=REG_TOP_SCRATCH_REG then
          cpu_top_rd_data <= scratch_data;
        end if;

        if cpu_address(11 downto 0)=REG_TOP_SERDES_LOOP then
          cpu_top_rd_data(rx_seriallpbken'range) <= rx_seriallpbken;
        end if;

        if cpu_address(11 downto 0)=REG_TOP_SERDES_LOCKTOREF then
          cpu_top_rd_data(rx_set_locktoref'range) <= rx_set_locktoref;
        end if;

        if cpu_address(11 downto 0)=REG_TOP_SERDES_LOCKTODATA then
          cpu_top_rd_data(rx_set_locktodata'range) <= rx_set_locktodata;
        end if;

        if cpu_address(11 downto 0)=REG_TOP_SERDES_STATUS_0 then
          cpu_top_rd_data(serdes_status(0)'range) <= serdes_status(0);
        end if;

        if cpu_address(11 downto 0)=REG_TOP_SERDES_STATUS_1 then
          cpu_top_rd_data(serdes_status(1)'range) <= serdes_status(1);
        end if;

        if cpu_address(11 downto 0)=REG_TOP_SERDES_STATUS_2 then
          cpu_top_rd_data(serdes_status(2)'range) <= serdes_status(2);
        end if;

        if cpu_address(11 downto 0)=REG_TOP_SERDES_STATUS_3 then
          cpu_top_rd_data(serdes_status(3)'range) <= serdes_status(3);
        end if;

        if cpu_address(11 downto 0)=REG_TOP_SERDES_RESET then
          cpu_top_rd_data(serdes_rst'range) <= serdes_rst;
        end if;

        -- These resets just toggle, write only
        if cpu_address(11 downto 0)=REG_TOP_JESD_RESET then
          cpu_top_rd_data(0) <= jesd_rx_reset;
          cpu_top_rd_data(1) <= jesd_tx_reset;
        end if;

        if cpu_address(11 downto 0)=REG_TOP_SYSTEM_PLL_STATUS then
          cpu_top_rd_data(0) <= system_pll_locked;
        end if;


        if cpu_address(11 downto 0)=REG_TOP_CPU_CLK_CNT then
          cpu_top_rd_data(cpu_clk_cnt'range) <= std_logic_vector(cpu_clk_cnt);
        end if;

        if cpu_address(11 downto 0)=REG_TOP_DEVICE_CLK_CNT then
          -- TBD GRAY CODE!!
          cpu_top_rd_data(device_clk_cnt'range) <= std_logic_vector(device_clk_cnt);
        end if;

        if cpu_address(11 downto 0)=REG_TOP_SYSREF_PERIOD then
          cpu_top_rd_data(sysref_cnt_max'range) <= sysref_cnt_max;
        end if;

        if cpu_address(11 downto 0)=REG_TOP_SYSREF_CTRL then
          cpu_top_rd_data(0) <= sysref_en;
          cpu_top_rd_data(1) <= sysref_out_en;
        end if;

        if cpu_address(11 downto 0)=REG_TOP_N_CHECK then
          cpu_top_rd_data(n_check'range) <= n_check;
        end if;

        if cpu_address(11 downto 0)=REG_TOP_RECONFIG_STATUS then
          cpu_top_rd_data(0) <= reconfig_tx_cal_busy;
          cpu_top_rd_data(0) <= reconfig_rx_cal_busy;
        end if;

        if cpu_address(11 downto 0)=REG_TOP_CAPTURE_CTRL then
          cpu_top_rd_data(0) <= capture_active_s1;
        end if;

        if cpu_address(11 downto 10)=REG_TOP_CAPTURE_RX(11 downto 10) then
          cpu_top_rd_data(cpu_capture_rd_data'range) <= cpu_capture_rd_data;
        end if;

      end if;
      spi_pending <= spi_pending and not spi_done;

      --! CPU WRITE !--
      if cpu_top_select='1' and cpu_write='1' and cpu_top_write_done='0' then

        if cpu_address(11 downto 0)=REG_TOP_SPI_RATE then
          spi_rate <= cpu_wr_data(spi_rate'range);
        end if;

        if cpu_address(11 downto 0)=REG_TOP_SPI_CONTROL then
          spi_wr_data <= cpu_wr_data(7 downto 0);
          spi_address <= cpu_wr_data(20 downto 8);
          spi_operation <= cpu_wr_data(24);
          spi_pending <= '1';
        end if;

        if cpu_address(11 downto 0)=REG_TOP_SAMPLE_CTRL then
          sample_type <= cpu_wr_data(2 downto 0);
          data_generator <= cpu_wr_data(8);
        end if;

        if cpu_address(11 downto 0)=REG_TOP_SCRATCH_REG then
          scratch_data <= cpu_wr_data;
        end if;

        if cpu_address(11 downto 0)=REG_TOP_SERDES_LOOP then
          rx_seriallpbken <= cpu_wr_data(rx_seriallpbken'range);
        end if;

        if cpu_address(11 downto 0)=REG_TOP_SERDES_LOCKTOREF then
          rx_set_locktoref <= cpu_wr_data(rx_set_locktoref'range);
        end if;

        if cpu_address(11 downto 0)=REG_TOP_SERDES_LOCKTODATA then
          rx_set_locktodata <= cpu_wr_data(rx_set_locktodata'range);
        end if;

        if cpu_address(11 downto 0)=REG_TOP_SERDES_RESET then
          serdes_rst <= cpu_wr_data(serdes_rst'range);
        end if;

        if cpu_address(11 downto 0)=REG_TOP_JESD_RESET then
          jesd_rx_reset <= cpu_wr_data(0);
          jesd_tx_reset <= cpu_wr_data(1);
        end if;

        if cpu_address(11 downto 0)=REG_TOP_DEVICE_RESET then
          cpu_reset_device <= cpu_wr_data(0);
        end if;

        if cpu_address(11 downto 0)=REG_TOP_SYSREF_PERIOD then
          sysref_cnt_max <= cpu_wr_data(sysref_cnt_max'range);
        end if;

        if cpu_address(11 downto 0)=REG_TOP_SYSREF_CTRL then
           sysref_en <= cpu_wr_data(0);
           sysref_out_en <= cpu_wr_data(1);
           sysref_toggle <= cpu_wr_data(2);
        end if;

        if cpu_address(11 downto 0)=REG_TOP_CAPTURE_CTRL then
          capture_toggle <= capture_toggle xor cpu_wr_data(0);
        end if;

        if cpu_address(11 downto 0)=REG_TOP_N_CHECK then
          n_check <= cpu_wr_data(n_check'range);
        end if;

      end if;

    end if;
  end process;

  -----------------------------------------------------------------------------
  -- SPI interface
  -----------------------------------------------------------------------------

  gen_spi:
  block
    signal spi_clk_cnt : std_logic_vector(3 downto 0);
    signal spi_clk_neg_edge : std_logic;
    signal spi_clk_pos_edge : std_logic;
    signal spi_cnt : std_logic_vector(4 downto 0);
    signal spi_active : std_logic;
  begin
    process(cpu_clk,cpu_reset)
    begin
      if cpu_reset='1' then
        spi_clk_cnt <= (others => '0');
        spi_clk_neg_edge <= '0';
        spi_clk_pos_edge <= '0';
        spi_cnt <= (others => '0');
        spi_active <= '0';
        spi_done <= '0';
        spi_do <= '0';
        spi_do_en <= '0';
        spi_cs_n <= '1';
        spi_clk <= '0';
        spi_rd_data <= (others => '0');
      elsif cpu_clk'event and cpu_clk='1' then
        spi_clk_cnt <= spi_clk_cnt + "1";
        spi_clk_neg_edge <= (spi_rate="00" and spi_clk_cnt(3 downto 0)="1111")
                         or (spi_rate="01" and spi_clk_cnt(2 downto 0)="111")
                         or (spi_rate="10" and spi_clk_cnt(1 downto 0)="11")
                         or (spi_rate="11" and spi_clk_cnt(0 downto 0)="1");
        spi_clk_pos_edge <= (spi_rate="00" and spi_clk_cnt(3 downto 0)="0111")
                         or (spi_rate="01" and spi_clk_cnt(2 downto 0)="011")
                         or (spi_rate="10" and spi_clk_cnt(1 downto 0)="01")
                         or (spi_rate="11" and spi_clk_cnt(0 downto 0)="0");
        if spi_clk_neg_edge='1' then
          spi_clk <= '0';
          if spi_active='0' then
            spi_cnt <= (others => '0');
            spi_active <= spi_pending;
            spi_done <= '0';
          else
            spi_cnt <= spi_cnt + "1";
            spi_active <= spi_cnt /= x"17";
            spi_done <= spi_cnt = x"17";
          end if;
          spi_do <= '0';
          if spi_cnt=0 then
            spi_do <= spi_operation;
          end if;
          for n in 0 to 12 loop
            if spi_cnt=n+3 then
              spi_do <= spi_address(12-n);
            end if;
          end loop;
          for n in 0 to 7 loop
            if spi_cnt=n+16 and spi_operation=SPI_OPERATION_WRITE then
              spi_do <= spi_wr_data(7-n);
            end if;
          end loop;
          spi_cs_n <= not spi_active;
          if spi_active='1' and (spi_cnt<16 or spi_operation=SPI_OPERATION_WRITE) then
            spi_do_en <= '1';
          else
            spi_do_en <= '0';
          end if;
        end if;
        if spi_clk_pos_edge='1' then
          spi_clk <= '1';
          if spi_cnt>=17 and spi_operation=SPI_OPERATION_READ then
            spi_rd_data(0) <= spi_di;
            spi_rd_data(7 downto 1) <= spi_rd_data(6 downto 0);
          end if;
        end if;
      end if;
    end process;
  end block;

  -----------------------------------------------------------------------------
  -- Sample TX (generation)
  -----------------------------------------------------------------------------
  process(clk_device,reset_device)
    variable data     : std_logic_vector(15 downto 0);
    variable shift    : std_logic_vector(15 downto 0);
    variable ctrl     : std_logic_vector(2 downto 0);
    variable cnt_data : array16(NO_CONVERTERS-1 downto 0);
  begin
    if reset_device=RESET_ACTIVE then
      sample_tx_data <= (others => '0');
      sample_tx_control_bits <= (others => '0');
      cnt_data := (others => (others => '0'));
    elsif clk_device'event and clk_device='1' then
      sample_tx_data <= (others => '0');
      for n in NO_CONVERTERS-1 downto 0 loop
        if sample_tx_valid(n)='1' then
          for i in 0 to NO_SERDES_WORDS-1 loop
            -- Choose sample bit size
            data := cnt_data(n)(15-log2_ceil(NO_SERDES_WORDS) downto 0) & conv_std_logic_vector(i,log2_ceil(NO_SERDES_WORDS));
            shift := (others => '0');
            for p in 0 to 15 loop
              for m in 0 to 8 loop
                if n_check = 16-m and p+m<=15 then
                  shift(p+m) := data(p);
                end if;
              end loop;
            end loop;
            sample_tx_data(16*(NO_SERDES_WORDS*n+i+1)-1 downto 16*(NO_SERDES_WORDS*n+i)) <= shift;
          end loop;
          cnt_data(n) := cnt_data(n) + 1;
        end if;
      end loop;
    end if;
  end process;

-----------------------------------------------------------------------------
  -- Sample RX (Check)
  -----------------------------------------------------------------------------

  sample_rx_check_clear <= not local_sync_b;

  process(clk_device,reset_device)
  begin
    if reset_device=RESET_ACTIVE then
      sample_check_incr <= (others => '0');
    elsif clk_device'event and clk_device='1' then
      sample_check_incr <= (others => '0');
      for n in 8 to 16 loop
        if n_check = n then
          sample_check_incr <= conv_std_logic_vector(2**(16-n),9);
        end if;
      end loop;
    end if;
  end process;

  proc_sample_rx:
  process(clk_device,reset_device)
    constant config_nt : integer := 16;
    type integer_array is array(integer range <>) of integer;
    variable data : std_logic_vector(15 downto 0);
    variable prbs_vector : std_logic_vector(NO_CONVERTERS*NO_SERDES_WORDS*16-1 downto 0);
    variable tmp16 : std_logic_vector(15 downto 0);
    variable data_expected : std_logic_vector(15 downto 0);
    variable ctrl : std_logic_vector(2 downto 0);
    variable cnt_data : array16(NO_CONVERTERS-1 downto 0);
    variable prbs     : std_logic_vector(22 downto 0);
    variable prbs_nxt : array23(NO_CONVERTERS-1 downto 0);
    variable new_prbs : std_logic_vector(22 downto 0);
    variable ok       : std_logic;
  begin
    if reset_device=RESET_ACTIVE then
      sample_rx_check_error <= (others => '0');
      sample_rx_check_valid <= (others => '0');
      cnt_data := (others => (others => '0'));
      prbs_reg <= (others => (others => '1'));
    elsif clk_device'event and clk_device='1' then
      sample_rx_check_error <= (others => '0');
      sample_rx_check_valid <= (others => '0');
      prbs_nxt              := prbs_reg;
      prbs_vector           := (others => '0');
      for n in 0 to NO_CONVERTERS-1 loop
        if sample_rx_check_clear='1' then
          cnt_data(n) := (others => '0');
          prbs_nxt := (others => (others => '1'));
        elsif sample_rx_valid(n)='1' then
          sample_rx_check_valid(n) <= '1';
          for i in 0 to NO_SERDES_WORDS-1 loop
            -- Choose sample bit size
            data := sample_rx_data(16*(NO_SERDES_WORDS*n+i+1)-1 downto 16*(NO_SERDES_WORDS*n+i));
            ctrl := "000";
            sample_rx_data_array(n*NO_SERDES_WORDS+i) <= data;
            sample_rx_ctrl_array(n*NO_SERDES_WORDS+i) <= ctrl;
            if sample_type=INCR and data/=cnt_data(n) then
              sample_rx_check_error(n) <= '1';
            end if;
            cnt_data(n) := data  + sample_check_incr;
          end loop;
        end if;
      end loop;
      if sample_rx_check_valid/=0  and sample_type/=INCR then
        if data_generator=ADI then
          for n in 0 to NO_CONVERTERS-1 loop
            for i in 0 to NO_SERDES_WORDS-1 loop
              prbs_vector(prbs_vector'high-n*16-i*32 downto prbs_vector'high-n*16-i*32-15) :=
                sample_rx_data_array(n*NO_SERDES_WORDS+i);
            end loop;
          end loop;
          case sample_type is
            when PRBS7   =>
              prbs := prbs_nxt(0);
              check_inv_prbs_flex(  d  => prbs_vector,
                                    i  => prbs,
                                    num => 16*NO_SERDES_WORDS*NO_CONVERTERS,
                                    pattern => 7,
                                    q  => new_prbs,
                                    ok => ok);
              sample_rx_check_error(NO_CONVERTERS-1 downto 0) <= (others => not ok);
              prbs_nxt(0) := new_prbs;
            when PRBS9   =>
              prbs := prbs_nxt(0);
              check_inv_prbs_flex(  d  => prbs_vector,
                                    i  => prbs,
                                    num => 16*NO_SERDES_WORDS*NO_CONVERTERS,
                                    pattern => 9,
                                    q  => new_prbs,
                                    ok => ok);
              sample_rx_check_error(NO_CONVERTERS-1 downto 0) <= (others => not ok);
              prbs_nxt(0) := new_prbs;
            when PRBS15  =>
              prbs := prbs_nxt(0);
              check_inv_prbs_flex(  d  => prbs_vector,
                                    i  => prbs,
                                    num => 16*NO_SERDES_WORDS*NO_CONVERTERS,
                                    pattern => 15,
                                    q  => new_prbs,
                                    ok => ok);
              sample_rx_check_error(NO_CONVERTERS-1 downto 0) <= (others => not ok);
              prbs_nxt(0) := new_prbs;
            when others  =>   -- PRBS23
              prbs := prbs_nxt(0);
              check_inv_prbs_flex(  d  => prbs_vector,
                                    i  => prbs,
                                    num => 16*NO_SERDES_WORDS*NO_CONVERTERS,
                                    pattern => 23,
                                    q  => new_prbs,
                                    ok => ok);
              sample_rx_check_error(NO_CONVERTERS-1 downto 0) <= (others => not ok);
              prbs_nxt(0) := new_prbs;
          end case;
        else
          -- Only for 14 bit samples!
          for n in 0 to NO_CONVERTERS-1 loop
            for i in 0 to NO_SERDES_WORDS-1 loop
              prbs_vector(prbs_vector'high-i*14 downto prbs_vector'high-i*14-13) :=
                sample_rx_data_array(n*NO_SERDES_WORDS+i)(15 downto 2);
            end loop;
            case sample_type is
              when PRBS7   =>
                prbs := prbs_nxt(n);
                check_inv_prbs_flex(  d  => prbs_vector,
                                      i  => prbs,
                                      num => 14*NO_SERDES_WORDS,
                                      pattern => 7,
                                      q  => new_prbs,
                                      ok => ok);
                sample_rx_check_error(n) <= not ok;
                prbs_nxt(n) := new_prbs;
              when PRBS9   =>
                prbs := prbs_nxt(n);
                check_inv_prbs_flex(  d  => prbs_vector,
                                      i  => prbs,
                                      num => 14*NO_SERDES_WORDS,
                                      pattern => 9,
                                      q  => new_prbs,
                                      ok => ok);
                sample_rx_check_error(n) <= not ok;
                prbs_nxt(n) := new_prbs;
              when PRBS15  =>
                prbs := prbs_nxt(n);
                check_inv_prbs_flex(  d  => prbs_vector,
                                      i  => prbs,
                                      num => 14*NO_SERDES_WORDS,
                                      pattern => 15,
                                      q  => new_prbs,
                                      ok => ok);
                sample_rx_check_error(n) <= not ok;
                prbs_nxt(n) := new_prbs;
              when others  =>   -- PRBS23
                prbs := prbs_nxt(n);
                check_inv_prbs_flex(  d  => prbs_vector,
                                      i  => prbs,
                                      num => 14*NO_SERDES_WORDS,
                                      pattern => 23,
                                      q  => new_prbs,
                                      ok => ok);
                sample_rx_check_error(n) <= not ok;
                prbs_nxt(n) := new_prbs;
            end case;
          end loop;
        end if;
      end if;
      prbs_reg <= prbs_nxt;
    end if;
  end process;


  -------------------------------------------------------------------
  -- Sample capture
  -------------------------------------------------------------------

  sample_capture:
  block
    constant BUF_AW       : integer := 5;
    constant SERDES_AW    : integer := log2_ceil(NO_SERDES_WORDS);
    constant CONVERTER_AW : integer := log2_ceil(NO_CONVERTERS);
    component buf_ram
      generic (
        DW    : integer;
        AW    : integer;
        DEPTH : integer);
      port (
        wr_clk  : in  std_logic;
        wr_en   : in  std_logic;
        wr_addr : in  std_logic_vector(AW-1 downto 0);
        wr_data : in  std_logic_vector(DW-1 downto 0);
        rd_clk  : in  std_logic;
        rd_en   : in  std_logic;
        rd_addr : in  std_logic_vector(AW-1 downto 0);
        rd_data : out std_logic_vector(DW-1 downto 0));
    end component;
    signal capture_toggle_s0 : std_logic;
    signal capture_toggle_s1 : std_logic;
    signal capture_toggle_s2 : std_logic;
    signal buf_wr_en         : std_logic;
    signal buf_wr_address    : std_logic_vector(BUF_AW-1 downto 0);
    signal capture_cnt       : std_logic_vector(BUF_AW-1 downto 0);
    type buf_capture_rd_data_type is array(0 to NO_CONVERTERS-1) of std_logic_vector(16*NO_SERDES_WORDS-1 downto 0);
    signal buf_capture_rd_data : buf_capture_rd_data_type;
    signal buf_converter_rd_data : std_logic_vector(16*NO_SERDES_WORDS-1 downto 0);
  begin
    process(clk_device,reset_device)
    begin
      if reset_device=RESET_ACTIVE then
        capture_toggle_s0 <= '0';
        capture_toggle_s1 <= '0';
        capture_toggle_s2 <= '0';
        capture_active <= '0';
        capture_cnt <= (others => '0');
        buf_wr_en <= '0';
        buf_wr_address <= (others => '0');
      elsif clk_device'event and clk_device='1' then
        capture_toggle_s0 <= capture_toggle;
        capture_toggle_s1 <= capture_toggle_s0;
        capture_toggle_s2 <= capture_toggle_s1;
        -- It is expected that samples will be valid simultanously
        capture_active <= capture_active or (capture_toggle_s2 xor capture_toggle_s1);
        buf_wr_en <= '0';
        if capture_active='1' then
          if sample_rx_valid(0)='1' then
            buf_wr_en <= '1';
            buf_wr_address <= capture_cnt;
            capture_cnt <= capture_cnt + "1";
            if and_all(capture_cnt)='1' then
              capture_active <= '0';
            end if;
          end if;
        else
          capture_cnt <= (others => '0');
        end if;
      end if;
    end process;
    gen_buf_capture_rx:
    for n in NO_CONVERTERS-1 downto 0 generate
      signal buf_wr_data : std_logic_vector(16*NO_SERDES_WORDS-1 downto 0);
    begin
      process(clk_device)
      begin
        if clk_device'event and clk_device='1' then
          buf_wr_data <= sample_rx_data(16*NO_SERDES_WORDS*(n+1)-1 downto 16*NO_SERDES_WORDS*n );
        end if;
      end process;
      buf_capture_rx: buf_ram
        generic map (
          DW    => 16*NO_SERDES_WORDS,
          AW    => BUF_AW,
          DEPTH => 0 )
        port map (
          wr_clk  => clk_device,
          wr_en   => buf_wr_en,
          wr_addr => buf_wr_address,
          wr_data => buf_wr_data,
          --
          rd_clk  => cpu_clk,
          rd_en   => '1',
          rd_addr => cpu_address(BUF_AW+SERDES_AW+1 downto SERDES_AW+2),
          rd_data => buf_capture_rd_data(n) );
    end generate;
    process(cpu_clk)
    begin
      if cpu_clk'event and cpu_clk='1' then
        buf_converter_rd_data <= buf_capture_rd_data(conv_integer(unsigned(cpu_address(BUF_AW+SERDES_AW+CONVERTER_AW+1 downto BUF_AW+SERDES_AW+2))));
        cpu_capture_rd_data <= (others => '0');
        for n in 0 to NO_SERDES_WORDS-1 loop
          if cpu_address(SERDES_AW+1 downto 2)=n then
            for i in 15 downto 0 loop
              cpu_capture_rd_data(i) <= buf_converter_rd_data(n*16+i);
            end loop;
          end if;
        end loop;
      end if;
    end process;
  end block;

  -------------------------------------------------------------------
  -- SYSREF synchronization counter
  -------------------------------------------------------------------

  synchronization:
  block
    signal sysref_en_s0 : std_logic;
    signal sysref_en_s1 : std_logic;
    signal sysref_toggle_s0 : std_logic;
    signal sysref_toggle_s1 : std_logic;
    signal sysref_toggle_s2 : std_logic;
    signal sysref_toggle_s3 : std_logic;
    signal sysref_cnt : std_logic_vector(15 downto 0);
  begin
    process(clk_device,reset_device)
    begin
      if reset_device=RESET_ACTIVE then
        sysref_en_s0 <= '0';
        sysref_en_s1 <= '0';
        sysref_toggle_s0 <= '0';
        sysref_toggle_s1 <= '0';
        sysref_toggle_s2 <= '0';
        sysref_toggle_s3 <= '0';
        sysref_cnt <= (others => '0');
        local_sysref <= '0';
      elsif clk_device'event and clk_device='1' then
        sysref_toggle_s0 <= sysref_toggle;
        sysref_toggle_s1 <= sysref_toggle_s0;
        sysref_toggle_s2 <= sysref_toggle_s1;
        sysref_en_s0 <= sysref_en;
        sysref_en_s1 <= sysref_en_s0;
        if sysref_en_s1='0' then
          sysref_cnt <= (others => '0');
          local_sysref <= (sysref_toggle_s3 xor sysref_toggle_s2) OR sysref_in;
          sysref_toggle_s3 <= sysref_toggle_s2;
        elsif sysref_cnt=sysref_cnt_max then
          sysref_cnt <= (others => '0');
          local_sysref <= sysref_out_en or (sysref_toggle_s3 xor sysref_toggle_s2);
          sysref_toggle_s3 <= sysref_toggle_s2;
        else
          sysref_cnt <= sysref_cnt + "1";
          local_sysref <= '0';
        end if;
      end if;
    end process;
    sync_b_out <= local_sync_b;
    sysref_out <= local_sysref;
  end block;

  -------------------------------------------------------------------
  -- Device reset
  -- Controlled from register
  -------------------------------------------------------------------
  process(clk_device,cpu_reset)
  begin
    if cpu_reset='1' then
      reset_device_s0 <= RESET_ACTIVE;
      reset_device_s1 <= RESET_ACTIVE;
      reset_device    <= RESET_ACTIVE;
    elsif rising_edge(clk_device) then
      reset_device_s0 <= cpu_reset_device;
      reset_device_s1 <= reset_device_s0;
      reset_device    <= reset_device_s1;
    end if;
  end process;


  -------------------------------------------------------------------
  -- Debug
  -------------------------------------------------------------------
  process(cpu_clk,cpu_reset)
  begin
    if cpu_reset='1' then
      cpu_clk_cnt <= (others => '0');
    elsif rising_edge(cpu_clk) then
      cpu_clk_cnt <= cpu_clk_cnt + 1;
    end if;
  end process;

  process(clk_device,reset_device)
  begin
    if reset_device='1' then
      device_clk_cnt <= (others => '0');
    elsif rising_edge(clk_device) then
      device_clk_cnt <= device_clk_cnt + 1;
    end if;
  end process;

--  gen_sigtap:
--  if not SIMULATION generate
--    sigtap_debug(0)               <= cpu_reset;
--    sigtap_debug(1)               <= jesd_rx_reset;
--    sigtap_debug(2)               <= jesd_tx_reset;
--    --sigtap_debug(6 downto 3)      <= serdes_rst;
--    --sigtap_debug(11 downto 8)     <= tx_std_pcfifo_full;
--    --sigtap_debug(15 downto 12)    <= tx_std_pcfifo_empty;
--    --sigtap_debug(19 downto 16)    <= rx_std_pcfifo_full;
--    --sigtap_debug(23 downto 20)    <= rx_std_pcfifo_empty;
--    sigtap_debug(93)              <= system_pll_locked;
--
----     SignalTap_128_inst : SignalTap_128
----     port map (
----       acq_clk          => cpu_clk,
----       acq_data_in      => sigtap_debug,
----       acq_trigger_in   => sigtap_debug
----     );
--  end generate;

  gen_no_sigtap:
  if SIMULATION generate
    sigtap_debug <= (others => '0');
  end generate;



end rtl;
