namespace eval boardInit {

    variable initialized 0
    variable errors 0
    variable sofFilename "../output_files/jesd_eval_hsmc_c5_enc_time_limited.sof"
    variable jdiFilename "../output_files/jesd_eval_hsmc_c5_enc.jdi"
    variable qsys_headerFilename "headers/soc_system.tcl"
    variable masterPath ""
    variable bytestreamPath ""
    variable jtaguartPath ""
    variable dash_path ""
    variable dashboardActive 0

    proc init {} {

        if { ${::boardInit::initialized} == 1 } {
            return -code ok "already initialized"
        }

        set ::boardInit::errors 0


        #
        # make sure that SoCKit board appears to be the only board available
        #
        if { [ llength [ get_service_paths device ] ] > 1 } {
            set ::boardInit::errors 1
            ::boardInit::updateDashboard
            return -code error "too many devices located, expecting only 1 SoCKit board"
        }

        if { [ llength [ get_service_paths device ] ] == 0 } {
            set ::boardInit::errors 1
            ::boardInit::updateDashboard
            return -code error "no devices located, expecting only 1 SoCKit board"
        }

        if { [ lsearch [ get_service_paths device ] "*5CSE*" ] < 0 } {
            set ::boardInit::errors 1
            ::boardInit::updateDashboard
            return -code error "the one device available does not appear to be a SoCKit"
        }

        #
        # setup the service path variables
        #
        if { [ lsearch [ get_service_paths master ] "*/phy_1/master" ] < 0 } {
            set ::boardInit::errors 1
            ::boardInit::updateDashboard
            return -code error "master service path was not identified correctly"
        }

        if { [ lsearch [ get_service_paths bytestream ] "*/jtaguart_0" ] < 0 } {
            set ::boardInit::errors 1
            ::boardInit::updateDashboard
            return -code error "jtaguart service path was not identified correctly"
        }

      
        set ::boardInit::masterPath     [ lindex [ get_service_paths master ] [ lsearch [ get_service_paths master ] "*/phy_1/master" ] ]
        set ::boardInit::jtaguartPath    [ lindex [ get_service_paths bytestream ] [ lsearch [ get_service_paths bytestream ] "*/jtaguart_0" ] ]

        #
        # load the Qsys header namespace if it isn't already loaded
        #
        if { ![ namespace exists ::QSYS_HEADER ] } {
            if { ![ file isfile ${::boardInit::qsys_headerFilename} ] } {
                set ::boardInit::errors 1
                ::boardInit::updateDashboard
                return -code error "unable to locate Qsys header file"
            }
            namespace eval :: {
                source ${::boardInit::qsys_headerFilename}
            }
            if { ![ namespace exists ::QSYS_HEADER ] } {
                set ::boardInit::errors 1
                ::boardInit::updateDashboard
                return -code error "Qsys header did not load properly"
            }
        }

        set ::boardInit::initialized 1
        return -code ok
    }

}
